




"""
Provide interface to Google App Engine Storage
Support for making credentials available on all devices and browsers (permanent storage)
We support storing user credentials in AppEngine Datastorage for permanent access from all devices and browsers.
Implementation uses the unique identity property of a Google User Account (".user_id()") to keep track
of which credentials belong to who.
"""

from google.appengine.ext import db, ndb		# Permanent storage
from google.appengine.api import memcache   # Temporary storage




def gae_sso_safe_key(user):
    """
    Constructs a Google App Engine Datastore Key to a Credentials Safe.
    Key is based on the user id of a Google Account. 
    This key is used to store generated cloud service credentials using
    Google Account as the identifier. 
    """
    return ndb.Key('GAESafe', user.user_id())



def _linkedin_sso_safe_key(user):
	"""
	This would be used if a LinkedIn Single Sign On mechanism is used,
	in that case the user id of a LinkedIn account is used as 
	base.
	"""
	raise RuntimeError('Not yet implemented, requires LinkedIn SSO support.')




def get_service_credential(service_name, user):
	"""
	Get existing entry / crete new for Google App Engine Datastore (permanent)
	@param service_name: Name of cloud service to get credentials for
	@type service_name: '<Entry in CLOUD_SERVICES>'
	@param user: Authenticated user (e.g. Google account)
	@return: <instance of ServiceCredentials> | None
	"""
	ancestor_key = gae_sso_safe_key(user)
	
	gae_creds = ServiceCredentials.get_from_safe(ancestor_key).fetch() # Fetching all credentials for the user (form safe)
	gae_creds = [cred for cred in gae_creds if cred.service=='PROJECTPLACE'] # Fetch for Projectplace 
	if gae_creds:
		return gae_creds[0] # We only support one entry per service
	return None


def create_service_credential(service_name, user):
	ancestor_key = gae_sso_safe_key(user)
	gae_cred = ServiceCredentials(parent=ancestor_key)
	gae_cred.service = service_name
	return gae_cred



CLOUD_SERVICES = dict()
CLOUD_SERVICES['pp'] = 'PROJECTPLACE'

"""
Model database structures

Sources of inspiration (thanks!):
- Python appengine_oauth_library

"""

class ServiceCredentials(ndb.Expando):
#class ServiceCredentials(db.Model):

	""" Models individual OAuth credential """
	service = ndb.StringProperty(required=True) 			# What cloud service this (oauth) credentials is for
	consumer_key = ndb.StringProperty(required=True)		# Client/Application/Consumer token used when getting token&secret
	consumer_secret = ndb.StringProperty(required=True)
	oauth_token = ndb.StringProperty()			# User specific token
	oauth_token_secret = ndb.StringProperty()			# User specific secret
	when_first_created = ndb.DateTimeProperty(auto_now_add=True) 	# When was credentials generated
	when_expires = ndb.DateTimeProperty()

	@classmethod
	def get_from_safe(cls, ancestor_key):
		return cls.query(ancestor=ancestor_key)

	def set_values(self, values):
		self.consumer_key = values['consumer_key']
		self.consumer_secret = values['consumer_secret']
		self.oauth_token = values['oauth_token']
		self.oauth_token_secret = values['oauth_token_secret']

	
	def clear(self):
		"""
		Clear outs all values from self that are not required/mandatory.
		"""
		keys = [key for key in self._values.keys() if key not in ['service', 'consumer_key', 'consumer_secret']]
		for key in keys:
			del self._values[key]



#**********************************************************************************************
#
#	Temporary data storage (in memcache) to support the oauth dance 
#
#**********************************************************************************************


def _gen_memcache_auth_key(service_name, auth_token):
	return "oauth_%s_%s" % (service_name, auth_token)

def save_temporary_auth_secret(service_name, auth_token, auth_secret):
	"""
	Saves the temporary auth token and secret (done before redirecting
		user to could service for login/granting of access to service)
	"""
	key = _gen_memcache_auth_key(service_name=service_name, auth_token=auth_token)
	memcache.set(key, auth_secret, time=20*60)

def get_temporary_auth_secret(service_name, auth_token):
	"""
	Get temporary auth secret
	"""
	key = _gen_memcache_auth_key(service_name=service_name, auth_token=auth_token)
	return memcache.get(key)

