#@param conn: Connection (includign authentication info) to API
#@type conn: <instance of requests_oauthlib.Oauth1Session>
conn = None


API_ENDPOINT = 'https://api.projectplace.com'


from os.path import dirname, join as joinpath
default_confpath = joinpath(dirname(__file__),'apiaccess.cfg')

from ppapiaccess.credentials import connect

import GenericDataContainer



class Project(GenericDataContainer.GenericDataContainer):
    def __init__(self, name=None, id=None, data=None):
        """
        Corresponds to a project in Projectplace
        @param name: Name of Project
        @type name: '<name>'
        @param id: Id of Project (if None then will be taken from "data" or fetched from Projectplace)
        @type id: <id>
        @param data: Data representing a project. None to fetch from API, {} to not fetch.
        @type data: {'name':'<name>','id':<id>, } 
        """
        
        if name==None and id==None:
            raise ValueError('"name" or "id" must be provided')


        if data==None:
            # Regarding data: We will fetch if data is None. In case of an empty dict
            # we do not fetch (assumes there is no need/use for the data). This can
            # be utilized to speed up init process. If you have no
            # need for the metadata (description, etc.) but are
            # interested in Boards & Cards methods then use data={}
            
            # Fetch meta data for Project from API.
            data = _get_projects_data(name=name, id=id)
            
            if not data:
                errormsg = """No Project could be fetched from Projectplace.com. \
Neither supplied name ("%s") or id ("%s") correspond to any \
accessible Project. Please verify these values and try again."""%(name, id)
                raise ValueError(errormsg)
            
            data = data[0]
            if not id:
                id = data['id']
            if not name:
                name = data['name']
            
        self.name = name
        self._id = id

        if data:
            # Populate current object with attributes corresponding to content in data
            data2 = data.copy()
            
            if not self._id:
                data2['_id'] = data2['id']
            del data2['id']     # id should not end up in self.id (it's stored in self._id)      
            
            if self.name:   
                del data2['name'] # We don't want to overwrite self.name (provided they have a value)
        
            # Call parent                        
            super(Project, self).__init__(initdict=data2)
        
        self._members = None

    
    @property
    def id(self):
        """
        Return project id
        @return: <number>
        """
        if not self._id:
            # Fetch data from PP API, force by providing None for data parameter
            self.__init__(self.name, id=None, data=None)
        return self._id 


    @property
    def members(self):
        """
        Return members associated with project
        @return [<instance of Person>,]
        """
        if self._members == None:
            # Members info not yet retrieved from API, do it
            import json

            self._members = dict()
            
            # Fetch data from API
            conn = connect()
            url = '%s/1/project/%d/members'%(API_ENDPOINT, self.id)
            response = conn.get(url)
            if response.status_code != 200:
                raise RuntimeError('Error occured. Server returned http status code %d. Return message: %s. URL: %s'%(response.status_code, response.text, url))

            # Store data as instances of class Person 
            for entry in json.loads(response.content):
                new_member = Person(id=entry['id'], name=entry['name'], data=entry)
                self._members[new_member.id] = new_member
                
        return self._members



class Person(GenericDataContainer.GenericDataContainer):
    service_host_url = 'https://service.projectplace.com'

    def __init__(self, name, id, data=None):
        """
        Corresponds to a Person in Projectplace
        @param name: Name of person
        @type name: '<name>'
        @param id: Id of person
        @type id: <id>
        @param data: Data representing a Person (e.g. user) all will be added to self.__dict__
        @type data: {'name':'<name>','id':<id>, ...} [primary data, more will be available]
        """
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            if data2.has_key('id'):
                del data2['id']     # We don't want these to overwrite the self.id
            if data2.has_key(name):
                del data2['name']   # We don't want these to overwrite the self.id
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2) # Set all content to attributes on self

    @property
    def image_url(self):
        """
        Return url to the persons's avatar.
        @return: '<url>'
        """
        return '%s%s'%(self.__class__.service_host_url, self.avatar) 

    @property
    def page_url(self):
        """
        Return url to a person's info page
        @return: '<url>'
        """
        return '%s/pp/pp.cgi/%s'%(self.__class__.service_host_url, self.id)

    
    



def _get_projects_data(name=None, id=0):
    """
    Get project data from Projectplace.com 
    @param name: Optional Project name, if provided will only return corresponding project
    @type name: '<name>'
    @param id: Optional Project id, if provided will only return corresponding project
    @type id: <id>
    @return: [{project data},]
    """
    from ppapiaccess import connect, conn, API_ENDPOINT
    import json

    connect()

    url = '%s/1/user/me/projects'%API_ENDPOINT
    response = conn.get(url)
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Returned data: %s'%(response.status_code, response.content))

    data = json.loads(response.content)
    
    if name or id:
        # Find the requested project
        if name:
            name = name.lower()
        else:
            name = ''

        for entry in data:
            if entry['name'].lower() == name or entry['id'] == id:
                return [entry]
                
        return list()
    else:
        return data
        


def get_projects(name=None, id=0, custom_class=None):
    """
    Get available projects at Projectplace.com 
    @param name: Optional Project name, if provided will only return corresponding project
    @type name: '<name>'
    @param id: Optional Project id, if provided will only return corresponding project
    @type id: <id>
    @param custom_class: A custom class to populate with project data
    @type custom_class: <subclass of ppapiaccess.Project>
    @return: [<instance of Project|custom_class>,]
    """
    data = _get_projects_data(name=name, id=id)
    if not data:
        return None
    
    entry = data[0]
    if not custom_class:
        return [Project(id=entry['id'], name=entry['name'], data=entry) for entry in _get_projects_data(name=name, id=id)]
    else:
        return [custom_class(id=entry['id'], name=entry['name'], data=entry) for entry in _get_projects_data(name=name, id=id)]
    

def get_me(creds=None):
    """
    Return basic profile info from Projectplace.com about the user account used to access API
    @return: <instance of ppapiaccess.Person>
    """
    from ppapiaccess import connect
    import json
    
    global conn
    if not conn: conn = connect(credentials=creds)

    global API_ENDPOINT
    
    url = '%s/1/user/me/profile'%API_ENDPOINT
    response = conn.get(url)
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Returned data: %s'%(response.status_code, response.content))
    data = json.loads(response.content)
    name = '%s %s'%(data['first_name'], data['last_name'])
    return Person(id=data['id'], name=name.strip(), data=data)







