def credentials_from_cfg(filepath):
    """
    Will extract credentials from specified configuration file. File should have a
    standard key=value format. The following keys will be extracted:
    consumer_key (mandatory)
    consumer_secret (mandatory)
    access_token_secret (optional)
    access_token_key (optional
    
    @param filepath: Path to configuration file
    @type filepath: '<file path>'
    @return: {Configurations}
    """
    import ConfigParser
    settings = dict()
    
    config = ConfigParser.ConfigParser()
    config.read(filepath)

    settings['consumer_key'] = config.get('Basic API configuration','consumer_key')
    settings['consumer_secret'] = config.get('Basic API configuration','consumer_secret')

    value = config.get('User Credentials','oauth_token_secret')
    if value:
        settings['oauth_token_secret'] = value

    value = config.get('User Credentials','oauth_token')
    if value:
        settings['oauth_token'] = value

    return settings



def minimum_cred_provided(creds):
    """
    Verify that minimum credential info is provided
    @param creds: Credentials info
    @type creds: {'consumer_key':'<value>','consumer_secret':'<value>'}
    """
    if not 'consumer_key' in creds or not creds['consumer_key']:
        raise ValueError('Missing mandatory key "consumer_key" in parameter "credentials"')
    elif not 'consumer_secret' in creds or not creds['consumer_secret']:
        raise ValueError('Missing mandatory key "consumer_secret" in parameter "credentials"')

    

def connect(credentials=None, verbose=True):
    """
    Create session to Projectplace API. This method should be called with credentials
    before trying to access any data through the API to initiate a session (else those
    will fail). If the credentials only include consumer credentials (general) and not
    also user credentials (specific for user account at Projectplace.com) then a manual
    authentication process will be initiated (you want to avoid this).
    
    @param credentials: Contain credential details for authentication with Projectplace.com API
    @type credentials: {'consumer_key': 'CONSUMER_KEY',
                        'consumer_secret': 'CONSUMER_SECRET',
                        'oauth_token': 'OPTIONAL_VALID_ACCESS_TOKEN_KEY_FOR_USER',
                        'oauth_token_secret': 'OPTIONAL_VALID_ACCESS_TOKEN_SECRET_FOR_USER'
        }
    @param api_endpoint: Server endpoint for accessing API (used to override API_ENDPOINT)
    @type api_endpoint: '<https://api-server>'
    @param verbose: Output retrieved user specific credentials (token and secret)
    @type verbose: Boolean
    """
    from requests_oauthlib import OAuth1Session
    import ppapiaccess 
    
    if not ppapiaccess.conn:
        if not credentials:
            raise ValueError('Parameter credentials required for API initialization')
        else:
            minimum_cred_provided(credentials)

        
        client = credentials['consumer_key']
        client_secret = credentials['consumer_secret']

        if "oauth_token" in credentials and credentials["oauth_token"] and \
           "oauth_token_secret" in credentials and credentials["oauth_token_secret"]:
            # We have user specific credentials, use them to initiate the session
            # Use access token 
            
            key = credentials['oauth_token']
            secret = credentials['oauth_token_secret']
            ppapiaccess.conn = OAuth1Session(client, client_secret=client_secret, 
                            resource_owner_key=key, resource_owner_secret=secret)
        else:
            # We have to get user specific credentials
            # OAuth endpoints given in the Projectplace API documentation
            request_token_url = '%s/initiate'%ppapiaccess.API_ENDPOINT
            authorization_base_url = '%s/authorize'%ppapiaccess.API_ENDPOINT
            access_token_url = '%s/token'%ppapiaccess.API_ENDPOINT


            # 2. Fetch a request token
            ppapiaccess.conn = OAuth1Session(client, client_secret=client_secret)
            ppapiaccess.conn.fetch_request_token(request_token_url)


            # 3. Redirect user to Projectplace for authorization
            authorization_url = ppapiaccess.conn.authorization_url(authorization_base_url)

            print 
            print "Missing user specific credentials. Initializing process to collect these."
            print "1. Use url to authorize (by providing user details and granting permission)"
            print "   URL: %s"%authorization_url
            print "2. Page reloads, copy the (new) url from the address bar."
            
            ## Disabled automatic launch of web browser since this is not always desired 
            #import webbrowser
            #webbrowser.open(authorization_url)
            
            # 4. Get the authorization verifier code from the callback url
            redirect_response = raw_input("3. Paste the URL here:")
            ppapiaccess.conn.parse_authorization_response(redirect_response)


            # 5. Fetch the access token and store in credentials
            token = ppapiaccess.conn.fetch_access_token(access_token_url)
            credentials["oauth_token"] = token["oauth_token"]
            credentials["oauth_token_secret"] = token["oauth_token_secret"]
            if verbose:
                print
                print "Collected user credentials. Save these in your credentials config."
                print " - oauth_token: %s"%credentials["oauth_token"]
                print " - oauth_token_secret: %s"%credentials["oauth_token_secret"]

    return ppapiaccess.conn






